﻿using Microsoft.Extensions.Logging;
using SushiBarContracts.BusinessLogicsContracts;
using SushiBarContracts.ViewModels;

namespace SushiBar
{
    public partial class FormStoreReplenishment : Form
    {
        private readonly ILogger _logger;
        private readonly IStoreLogic _storeLogic;
        private readonly ISushiLogic _sushiLogic;
        private readonly List<StoreViewModel>? _listStores;
        private readonly List<SushiViewModel>? _listSushi;
        public FormStoreReplenishment(ILogger<FormStoreReplenishment> logger, IStoreLogic storeLogic, ISushiLogic sushiLogic)
        {
            InitializeComponent();
            _storeLogic = storeLogic;
            _sushiLogic = sushiLogic;
            _logger = logger;
            _listStores = storeLogic.ReadList(null);
            if (_listStores != null)
            {
                comboBoxStore.DisplayMember = "StoreName";
                comboBoxStore.ValueMember = "Id";
                comboBoxStore.DataSource = _listStores;
                comboBoxStore.SelectedItem = null;
            }

            _listSushi = sushiLogic.ReadList(null);
            if (_listSushi != null)
            {
                comboBoxSushi.DisplayMember = "SushiName";
                comboBoxSushi.ValueMember = "Id";
                comboBoxSushi.DataSource = _listSushi;
                comboBoxSushi.SelectedItem = null;
            }
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            if (comboBoxStore.SelectedValue == null)
            {
                MessageBox.Show("Select store", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (comboBoxSushi.SelectedValue == null)
            {
                MessageBox.Show("Select sushi", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Add sushi to store");

            try
            {
                var sushi = _sushiLogic.ReadElement(new()
                {
                    Id = (int)comboBoxSushi.SelectedValue
                });

                if (sushi == null)
                {
                    throw new Exception("Sushi is not found. Additional info below.");
                }

                var resultOperation = _storeLogic.SupplySushi(
                     model: new() { Id = (int)comboBoxStore.SelectedValue },
                     sushi: sushi,
                     quantity: Convert.ToInt32(textBoxCount.Text)
                );

                if (!resultOperation)
                {
                    throw new Exception("Error on saving. Additional info below.");
                }

                MessageBox.Show("Save is successful", "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error on saving");
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
