﻿using Microsoft.Extensions.Logging;
using SushiBarContracts.BindingModels;
using SushiBarContracts.BusinessLogicsContracts;

namespace SushiBar
{
    public partial class FormStoreSell : Form
    {
        private readonly ILogger _logger;
        private readonly ISushiLogic _logicSushi;
        private readonly IStoreLogic _logicStore;
        
        public FormStoreSell(ILogger<FormStoreSell> logger, ISushiLogic sushiLogic, IStoreLogic storeLogic)
        {
            InitializeComponent();
            _logger = logger;
            _logicSushi = sushiLogic;
            _logicStore = storeLogic;
        }

        private void ButtonSell_Click(object sender, EventArgs e)
        {
            if (Count.Value == 0)
            {
                MessageBox.Show("Count must be more then zero!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (SushiNames.SelectedValue == null)
            {
                MessageBox.Show("Select sushi", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Create sell");
            try
            {
                var operationResult = _logicStore.SellSushi(
                    new SushiBindingModel()
                    {
                        Id = Convert.ToInt32(SushiNames.SelectedValue)
                    },
                    (int)Count.Value
                );
                if (!operationResult)
                {
                    throw new Exception("Error on create sell. Additional info below");
                }
                MessageBox.Show("Complete create", "Info", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error on create");
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void FormStoreSell_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Load sushi to sell");
            try
            {
                var list = _logicSushi.ReadList(null);
                if (list == null) return;
                SushiNames.DisplayMember = "SushiName";
                SushiNames.ValueMember = "Id";
                SushiNames.DataSource = list;
                SushiNames.SelectedItem = null;

            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Error on loading");
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
