﻿using SushiBarFileImplement.Models;
using System.Xml.Linq;

namespace SushiBarFileImplement
{
    public class DataFileSingleton
    {
        private static DataFileSingleton? instance;
        private readonly string ComponentFileName = "Component.xml";
        private readonly string OrderFileName = "Order.xml";
        private readonly string SushiFileName = "Sushi.xml";
        private readonly string StoreFileName = "Store.xml";
        public List<Component> Components { get; }
        public List<Order> Orders { get; }
        public List<Sushi> Sushis { get; }
        public List<Store> Stores { get; }
        public static DataFileSingleton GetInstance()
        {
            instance ??= new DataFileSingleton();
            return instance;
        }
        public void SaveComponents() => SaveData(Components, ComponentFileName, "Components", x => x.GetXElement);
        public void SaveSushis() => SaveData(Sushis, SushiFileName, "Sushis", x => x.GetXElement);
        public void SaveOrders() => SaveData(Orders, OrderFileName, "Orders", x => x.GetXElement);
        public void SaveStores() => SaveData(Stores, StoreFileName, "Store", x => x.GetXElement);
        
        private DataFileSingleton()
        {
            Components = LoadData(ComponentFileName, "Component", x => Component.Create(x)!)!;
            Sushis     = LoadData(SushiFileName, "Sushi", x => Sushi.Create(x)!)!;
            Orders     = LoadData(OrderFileName, "Order", x => Order.Create(x)!)!;
            Stores = LoadData(StoreFileName, "Store", x => Store.Create(x)!)!;
        }
        private static List<T>? LoadData<T>(string filename, string xmlNodeName, Func<XElement, T> selectFunction)
        {
            return File.Exists(filename) ? XDocument.Load(filename)?.Root?.Elements(xmlNodeName)?.Select(selectFunction)?.ToList() : new List<T>();
        }
        private static void SaveData<T>(IReadOnlyCollection<T> data, string filename, string xmlNodeName, Func<T, XElement> selectFunction)
        {
            if (data != null)
            {
                new XDocument(new XElement(xmlNodeName, data.Select(selectFunction).ToArray())).Save(filename);
            }
        }
    }
}
